@@ -1,24 +1,22 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- *  test-widget.c
+ * test-widget.c
  *
- *  Copyright (C) 2001
- *  Mikael Hermansson<tyan@linux.se>
+ * Copyright (C) 2001 - Mikael Hermansson <tyan@linux.se>
+ * Copyright (C) 2003 - Gustavo Giráldez <gustavo.giraldez@gmx.net>
  *
- *  Copyright (C) 2003 - Gustavo Giráldez <gustavo.giraldez@gmx.net>
+ * This library is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Library General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 /* If TEST_XML_MEM is defined the test program will try to detect memory
@@ -36,6 +34,7 @@
 #include <gtksourceview/gtksourcelanguagemanager.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
 #include <gtksourceview/gtksourceprintcompositor.h>
+#include <gtksourceview/gtksourceiter.h>
 #ifdef TEST_XML_MEM
 #include <libxml/xmlreader.h>
 #endif
@@ -52,6 +51,10 @@ static GtkSourceStyleScheme *style_scheme = NULL;
 #define MARK_TYPE_1      "one"
 #define MARK_TYPE_2      "two"
 
+#if !GTK_CHECK_VERSION (2, 18, 0)
+#define gtk_widget_get_visible(widget)                          (GTK_WIDGET_VISIBLE (widget))
+#endif /* GTK+ < 2.18.0 */
+
 
 /* Private prototypes -------------------------------------------------------- */
 
@@ -59,7 +62,9 @@ static void       open_file_cb                   (GtkAction       *action,
 						  gpointer         user_data);
 static void       print_file_cb                  (GtkAction       *action,
 						  gpointer         user_data);
-static void       debug_thing_3_cb		 (GtkAction       *action,
+static void       find_cb			 (GtkAction       *action,
+						  gpointer         user_data);
+static void       replace_cb			 (GtkAction       *action,
 						  gpointer         user_data);
 
 static void       new_view_cb                    (GtkAction       *action,
@@ -89,6 +94,13 @@ static void       indent_toggled_cb              (GtkAction       *action,
 						  GtkAction       *current,
 						  gpointer         user_data);
 
+static void       forward_string_cb              (GtkAction       *action,
+						  gpointer         user_data);
+
+static void       backward_string_cb             (GtkAction       *action,
+						  gpointer         user_data);
+
+
 static GtkWidget *create_view_window             (GtkSourceBuffer *buffer,
 						  GtkSourceView   *from);
 
@@ -103,17 +115,23 @@ static GtkActionEntry buffer_action_entries[] = {
 };
 
 static GtkActionEntry view_action_entries[] = {
-	{ "FileMenu", NULL, "_File" },
+	{ "FileMenu", NULL, "_File", NULL, NULL, NULL },
 	{ "Print", GTK_STOCK_PRINT, "_Print", "<control>P",
 	  "Print the current file", G_CALLBACK (print_file_cb) },
-	{ "ViewMenu", NULL, "_View" },
+	{ "ViewMenu", NULL, "_View", NULL, NULL, NULL },
 	{ "NewView", GTK_STOCK_NEW, "_New View", NULL,
 	  "Create a new view of the file", G_CALLBACK (new_view_cb) },
-	{ "TabWidth", NULL, "_Tab Width" },
-	{ "IndentWidth", NULL, "I_ndent Width" },
-	{ "SmartHomeEnd", NULL, "_Smart Home/End" },
-	{ "DebugThing3", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
-	  "Search and Replace", G_CALLBACK (debug_thing_3_cb) },
+	{ "TabWidth", NULL, "_Tab Width", NULL, NULL, NULL },
+	{ "IndentWidth", NULL, "I_ndent Width", NULL, NULL, NULL },
+	{ "SmartHomeEnd", NULL, "_Smart Home/End", NULL, NULL, NULL },
+	{ "Find", GTK_STOCK_FIND, "_Find", "<control>F",
+	  "Find", G_CALLBACK (find_cb) },
+	{ "Replace", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
+	  "Search and Replace", G_CALLBACK (replace_cb) },
+	{ "ForwardString", NULL, "_Forward to string toggle", "<control>S",
+	  "Forward to the start or end of the next string", G_CALLBACK (forward_string_cb) },
+	{ "BackwardString", NULL, "_Backward to string toggle", "<control><shift>S",
+	  "Backward to the start or end of the next string", G_CALLBACK (backward_string_cb) }
 };
 
 static GtkToggleActionEntry toggle_entries[] = {
@@ -218,6 +236,9 @@ static const gchar *view_ui_description =
 "        <menuitem action=\"SmartHomeEndAfter\"/>"
 "        <menuitem action=\"SmartHomeEndAlways\"/>"
 "      </menu>"
+"      <separator/>"
+"      <menuitem action=\"ForwardString\"/>"
+"      <menuitem action=\"BackwardString\"/>"
 "    </menu>"
 "  </menubar>"
 "</ui>";
@@ -228,7 +249,8 @@ static const gchar *buffer_ui_description =
 "    <menu action=\"FileMenu\">"
 "      <menuitem action=\"Open\"/>"
 "      <menuitem action=\"Print\"/>"
-"      <menuitem action=\"DebugThing3\"/>"
+"      <menuitem action=\"Find\"/>"
+"      <menuitem action=\"Replace\"/>"
 "      <separator/>"
 "      <menuitem action=\"Quit\"/>"
 "    </menu>"
@@ -255,7 +277,7 @@ error_dialog (GtkWindow *parent, const gchar *msg, ...)
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
 					 GTK_BUTTONS_OK,
-					 tmp);
+					 "%s", tmp);
 	g_free (tmp);
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
@@ -613,24 +635,86 @@ new_view_cb (GtkAction *action, gpointer user_data)
 	gtk_widget_show (window);
 }
 
+static void
+forward_string_cb (GtkAction *action,
+		   gpointer   user_data)
+{
+	GtkSourceBuffer *buffer;
+	GtkSourceView *view;
+	GtkTextIter iter;
+	GtkTextMark *insert;
+
+	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
+
+	view = GTK_SOURCE_VIEW (user_data);
+	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+	insert = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (buffer));
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+	                                  &iter,
+	                                  insert);
+
+	if (gtk_source_buffer_iter_forward_to_context_class_toggle (buffer,
+	                                                            &iter,
+	                                                            "string"))
+	{
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (buffer), &iter);
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view), insert);
+	}
+}
+
+static void
+backward_string_cb (GtkAction *action,
+		    gpointer   user_data)
+{
+	GtkSourceBuffer *buffer;
+	GtkSourceView *view;
+	GtkTextIter iter;
+	GtkTextMark *insert;
+
+	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
+
+	view = GTK_SOURCE_VIEW (user_data);
+	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+	insert = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (buffer));
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+	                                  &iter,
+	                                  insert);
+
+	if (gtk_source_buffer_iter_backward_to_context_class_toggle (buffer,
+	                                                             &iter,
+	                                                             "string"))
+	{
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (buffer), &iter);
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view), insert);
+	}
+}
 
 /* Buffer action callbacks ------------------------------------------------------------ */
 
+static struct {
+	char *what;
+	char *replacement;
+	GtkSourceSearchFlags flags;
+} search_data = {
+	NULL,
+	NULL,
+	GTK_SOURCE_SEARCH_CASE_INSENSITIVE
+};
+
 static gboolean
-replace_dialog (GtkWidget *widget,
-		char     **what_p,
-		char     **replacement_p)
+search_dialog (GtkWidget            *widget,
+	       gboolean              replace,
+	       char                **what_p,
+	       char                **replacement_p,
+	       GtkSourceSearchFlags *flags_p)
 {
 	GtkWidget *dialog;
 	GtkEntry *entry1, *entry2;
-	static char *what, *replacement;
+	GtkToggleButton *case_sensitive;
 
-	if (!what)
-		what = g_strdup ("gtk");
-	if (!replacement)
-		replacement = g_strdup ("boo");
-
-	dialog = gtk_dialog_new_with_buttons ("Replace",
+	dialog = gtk_dialog_new_with_buttons (replace ? "Replace" : "Find",
 					      GTK_WINDOW (gtk_widget_get_toplevel (widget)),
 					      GTK_DIALOG_MODAL,
 					      GTK_STOCK_CANCEL,
@@ -642,18 +726,26 @@ replace_dialog (GtkWidget *widget,
 
 	entry1 = g_object_new (GTK_TYPE_ENTRY,
 			       "visible", TRUE,
-			       "text", what ? what : "",
+			       "text", search_data.what ? search_data.what : "",
 			       "activates-default", TRUE,
 			       NULL);
-	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-				     GTK_WIDGET (entry1));
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+				     GTK_WIDGET (entry1), TRUE, TRUE, 0);
 	entry2 = g_object_new (GTK_TYPE_ENTRY,
-			       "visible", TRUE,
-			       "text", replacement ? replacement : "",
+			       "visible", replace,
+			       "text", search_data.replacement ? search_data.replacement : "",
 			       "activates-default", TRUE,
 			       NULL);
-	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-				     GTK_WIDGET (entry2));
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+				     GTK_WIDGET (entry2), TRUE, TRUE, 0);
+
+	case_sensitive = g_object_new (GTK_TYPE_CHECK_BUTTON,
+				       "visible", TRUE,
+				       "label", "Case sensitive",
+				       "active", !(search_data.flags & GTK_SOURCE_SEARCH_CASE_INSENSITIVE),
+				       NULL);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+			    GTK_WIDGET (case_sensitive), FALSE, FALSE, 0);
 
 	while (TRUE)
 	{
@@ -667,47 +759,84 @@ replace_dialog (GtkWidget *widget,
 			break;
 	}
 
-	g_free (what);
-	*what_p = what = g_strdup (gtk_entry_get_text (entry1));
-	g_free (replacement);
-	*replacement_p = replacement = g_strdup (gtk_entry_get_text (entry2));
+	g_free (search_data.what);
+	*what_p = search_data.what = g_strdup (gtk_entry_get_text (entry1));
+	g_free (search_data.replacement);
+	*replacement_p = search_data.replacement = g_strdup (gtk_entry_get_text (entry2));
+	*flags_p = search_data.flags = gtk_toggle_button_get_active (case_sensitive) ?
+					0 : GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
 
 	gtk_widget_destroy (dialog);
 	return TRUE;
 }
 
 static void
-debug_thing_3_cb (GtkAction *action,
-		  gpointer   user_data)
+do_search_replace (GtkTextView *view,
+		   gboolean     replace)
 {
-	GtkTextView *view = user_data;
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
 	GtkTextIter iter;
 	char *what, *replacement;
+	GtkSourceSearchFlags flags;
 
-	if (!replace_dialog (GTK_WIDGET (view), &what, &replacement))
+	if (!search_dialog (GTK_WIDGET (view), replace, &what, &replacement, &flags))
 		return;
 
-	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
+	if (replace)
+	{
+		gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
 
-	while (TRUE)
+		while (TRUE)
+		{
+			GtkTextIter match_start, match_end;
+
+			if (!gtk_source_iter_forward_search (&iter, what, flags,
+							     &match_start,
+							     &match_end,
+							     NULL))
+			{
+				break;
+			}
+
+			gtk_text_buffer_delete (buffer, &match_start, &match_end);
+			gtk_text_buffer_insert (buffer, &match_start, replacement, -1);
+			iter = match_start;
+		}
+	}
+	else
 	{
 		GtkTextIter match_start, match_end;
 
-		if (!gtk_text_iter_forward_search (&iter, what, 0,
-						   &match_start,
-						   &match_end,
-						   NULL))
-		{
-			break;
-		}
+		gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
 
-		gtk_text_buffer_delete (buffer, &match_start, &match_end);
-		gtk_text_buffer_insert (buffer, &match_start, replacement, -1);
-		iter = match_start;
+		if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, NULL))
+		{
+			gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+		}
+		else
+		{
+			GtkTextIter insert = iter;
+			gtk_text_buffer_get_start_iter (buffer, &iter);
+			if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, &insert))
+				gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+		}
 	}
 }
 
+static void
+find_cb (GtkAction *action,
+	 gpointer   user_data)
+{
+	do_search_replace (user_data, FALSE);
+}
+
+static void
+replace_cb (GtkAction *action,
+	    gpointer   user_data)
+{
+	do_search_replace (user_data, TRUE);
+}
+
 static void
 open_file_cb (GtkAction *action, gpointer user_data)
 {
@@ -968,15 +1097,18 @@ static void
 update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 {
 	gchar *msg;
-	gint row, col, chars, tabwidth;
-	GtkTextIter iter, start;
+	gint row, chars;
+	guint col;
+	GtkTextIter iter;
 	GtkSourceView *view;
 	GtkLabel *pos_label;
+	gchar **classes;
+	gchar **classes_ptr;
+	GString *str;
 
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
 
 	view = GTK_SOURCE_VIEW (user_data);
-	tabwidth = gtk_source_view_get_tab_width (view);
 	pos_label = GTK_LABEL (g_object_get_data (G_OBJECT (view), "pos_label"));
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
@@ -985,26 +1117,32 @@ update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 
 	chars = gtk_text_iter_get_offset (&iter);
 	row = gtk_text_iter_get_line (&iter) + 1;
+	col = gtk_source_view_get_visual_column (view, &iter) + 1;
 
-	start = iter;
-	gtk_text_iter_set_line_offset (&start, 0);
-	col = 0;
+	classes = gtk_source_buffer_get_context_classes_at_iter (GTK_SOURCE_BUFFER (buffer),
+	                                                         &iter);
 
-	while (!gtk_text_iter_equal (&start, &iter))
+	str = g_string_new ("");
+	classes_ptr = classes;
+
+	while (classes_ptr && *classes_ptr)
 	{
-		if (gtk_text_iter_get_char (&start) == '\t')
+		if (classes_ptr != classes)
 		{
-			col += (tabwidth - (col % tabwidth));
+			g_string_append (str, ", ");
 		}
-		else
-			++col;
 
-		gtk_text_iter_forward_char (&start);
+		g_string_append_printf (str, "%s", *classes_ptr);
+		++classes_ptr;
 	}
 
-	msg = g_strdup_printf ("char: %d, line: %d, column: %d", chars, row, col);
+	g_strfreev (classes);
+
+	msg = g_strdup_printf ("char: %d, line: %d, column: %d, classes: %s", chars, row, col, str->str);
 	gtk_label_set_text (pos_label, msg);
-      	g_free (msg);
+
+	g_free (msg);
+	g_string_free (str, TRUE);
 }
 
 static void
@@ -1054,80 +1192,100 @@ window_deleted_cb (GtkWidget *widget, GdkEvent *ev, gpointer user_data)
 	return TRUE;
 }
 
-static gboolean
-button_press_cb (GtkWidget *widget, GdkEventButton *ev, gpointer user_data)
+static void
+line_mark_activated (GtkSourceGutter *gutter, 
+                     GtkTextIter     *iter,
+                     GdkEventButton  *ev,
+                     GtkSourceView   *view)
 {
-	GtkSourceView *view;
 	GtkSourceBuffer *buffer;
+	GSList *mark_list;
+	const gchar *mark_type;
 
-	g_return_val_if_fail (GTK_IS_SOURCE_VIEW (widget), FALSE);
+	if (ev->button == 1)
+		mark_type = MARK_TYPE_1;
+	else
+		mark_type = MARK_TYPE_2;
 
-	view = GTK_SOURCE_VIEW (widget);
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
-	if (!gtk_source_view_get_show_line_marks (view))
-		return FALSE;
+	/* get the marks already in the line */
+	mark_list = gtk_source_buffer_get_source_marks_at_line (buffer,
+								gtk_text_iter_get_line (iter),
+								mark_type);
 
-	/* check that the click was on the left gutter */
-	if (ev->window == gtk_text_view_get_window (GTK_TEXT_VIEW (view),
-						    GTK_TEXT_WINDOW_LEFT))
+	if (mark_list != NULL)
 	{
-		gint y_buf;
-		GtkTextIter line_start;
-		GSList *mark_list;
-		const gchar *mark_type;
-
-		if (ev->button == 1)
-			mark_type = MARK_TYPE_1;
-		else
-			mark_type = MARK_TYPE_2;
-
-		gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
-						       GTK_TEXT_WINDOW_LEFT,
-						       ev->x, ev->y,
-						       NULL, &y_buf);
-
-		/* get line bounds */
-		gtk_text_view_get_line_at_y (GTK_TEXT_VIEW (view),
-					     &line_start,
-					     y_buf,
-					     NULL);
-
-		/* get the marks already in the line */
-		mark_list = gtk_source_buffer_get_source_marks_at_line (buffer,
-									gtk_text_iter_get_line (&line_start),
-									mark_type);
-
-		if (mark_list != NULL)
-		{
-			/* just take the first and delete it */
-			gtk_text_buffer_delete_mark (GTK_TEXT_BUFFER (buffer),
-						     GTK_TEXT_MARK (mark_list->data));
-		}
-		else
-		{
-			/* no mark found: create one */
-			gtk_source_buffer_create_source_mark (buffer,
-							      NULL,
-							      mark_type,
-							      &line_start);
-		}
-
-		g_slist_free (mark_list);
+		/* just take the first and delete it */
+		gtk_text_buffer_delete_mark (GTK_TEXT_BUFFER (buffer),
+					     GTK_TEXT_MARK (mark_list->data));
+	}
+	else
+	{
+		/* no mark found: create one */
+		gtk_source_buffer_create_source_mark (buffer,
+						      NULL,
+						      mark_type,
+						      iter);
 	}
 
-	return FALSE;
+	g_slist_free (mark_list);
 }
 
 
 /* Window creation functions -------------------------------------------------------- */
 
+static gchar *
+mark_tooltip_func (GtkSourceMark *mark,
+		   gpointer	  user_data)
+{
+	GtkTextBuffer *buf;
+	GtkTextIter iter;
+	gint line, column;
+	
+	buf = gtk_text_mark_get_buffer (GTK_TEXT_MARK (mark));
+	
+	gtk_text_buffer_get_iter_at_mark (buf, &iter, GTK_TEXT_MARK (mark));
+	line = gtk_text_iter_get_line (&iter) + 1;
+	column = gtk_text_iter_get_line_offset (&iter);
+
+	if (strcmp (gtk_source_mark_get_category (mark), MARK_TYPE_1) == 0)
+		return g_strdup_printf ("Line: %d, Column: %d", line, column);
+	else
+		return g_strdup_printf ("<b>Line</b>: %d\n<i>Column</i>: %d", line, column);
+}
+
+static void
+add_source_mark_pixbufs (GtkSourceView *view)
+{
+	GdkColor color;
+
+	gdk_color_parse ("lightgreen", &color);
+	gtk_source_view_set_mark_category_background (view, MARK_TYPE_1, &color);
+	gtk_source_view_set_mark_category_icon_from_stock (view, MARK_TYPE_1, GTK_STOCK_YES);
+	gtk_source_view_set_mark_category_priority (view, MARK_TYPE_1, 1);
+	gtk_source_view_set_mark_category_tooltip_func (view,
+							MARK_TYPE_1,
+							mark_tooltip_func,
+							NULL,
+							NULL);
+
+	gdk_color_parse ("pink", &color);
+	gtk_source_view_set_mark_category_background (view, MARK_TYPE_2, &color);
+	gtk_source_view_set_mark_category_icon_from_stock (view, MARK_TYPE_2, GTK_STOCK_NO);
+	gtk_source_view_set_mark_category_priority (view, MARK_TYPE_2, 2);
+	gtk_source_view_set_mark_category_tooltip_markup_func (view,
+							       MARK_TYPE_2,
+							       mark_tooltip_func,
+							       NULL,
+							       NULL);
+}
+
 static GtkWidget *
 create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 {
 	GtkWidget *window, *sw, *view, *vbox, *pos_label, *menu;
 	PangoFontDescription *font_desc = NULL;
-	GdkPixbuf *pixbuf;
 	GtkAccelGroup *accel_group;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
@@ -1144,13 +1302,13 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 
 	/* view */
 	view = gtk_source_view_new_with_buffer (buffer);
-
+	
 	if (style_scheme)
 		gtk_source_buffer_set_style_scheme (buffer, style_scheme);
 
 	g_signal_connect (buffer, "mark-set", G_CALLBACK (move_cursor_cb), view);
 	g_signal_connect (buffer, "changed", G_CALLBACK (update_cursor_position), view);
-	g_signal_connect (view, "button-press-event", G_CALLBACK (button_press_cb), NULL);
+	g_signal_connect (view, "line-mark-activated", G_CALLBACK (line_mark_activated), view);
 	g_signal_connect (window, "delete-event", (GCallback) window_deleted_cb, view);
 
 	/* action group and UI manager */
@@ -1211,7 +1369,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 		gtk_widget_modify_font (view, font_desc);
 		pango_font_description_free (font_desc);
 	}
-
+	
 	/* change view attributes to match those of from */
 	if (from)
 	{
@@ -1262,40 +1420,7 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 		g_free (tmp);
 	}
 
-	/* add source mark pixbufs */
-	error = NULL;
-	if ((pixbuf = gdk_pixbuf_new_from_file (TOP_SRCDIR "/tests/gnome-gmush.png", &error)))
-	{
-		GdkColor color;
-		gdk_color_parse ("lightgreen", &color);
-		gtk_source_view_set_mark_category_background (GTK_SOURCE_VIEW (view), MARK_TYPE_1, &color);
-		gtk_source_view_set_mark_category_pixbuf (GTK_SOURCE_VIEW (view), MARK_TYPE_1, pixbuf);
-		gtk_source_view_set_mark_category_priority (GTK_SOURCE_VIEW (view), MARK_TYPE_1, 1);
-		g_object_unref (pixbuf);
-	}
-	else
-	{
-		g_message ("could not load source mark 1 image. Spurious messages might get triggered: %s\n",
-		error->message);
-		g_error_free (error);
-	}
-
-	error = NULL;
-	if ((pixbuf = gdk_pixbuf_new_from_file (TOP_SRCDIR "/tests/apple-red.png", &error)))
-	{
-		GdkColor color;
-		gdk_color_parse ("pink", &color);
-		gtk_source_view_set_mark_category_background (GTK_SOURCE_VIEW (view), MARK_TYPE_2, &color);
-		gtk_source_view_set_mark_category_pixbuf (GTK_SOURCE_VIEW (view), MARK_TYPE_2, pixbuf);
-		gtk_source_view_set_mark_category_priority (GTK_SOURCE_VIEW (view), MARK_TYPE_2, 2);
-		g_object_unref (pixbuf);
-	}
-	else
-	{
-		g_message ("could not load source mark 2 image. Spurious messages might get triggered: %s\n",
-		error->message);
-		g_error_free (error);
-	}
+	add_source_mark_pixbufs (GTK_SOURCE_VIEW (view));
 
 	gtk_widget_show_all (vbox);
 
@@ -1452,9 +1577,9 @@ main (int argc, char *argv[])
 	GtkSourceStyleSchemeManager *sm;
 	GtkSourceBuffer *buffer;
 
-	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/gtksourceview/language-specs", NULL};
-	gchar *builtin_sm_dirs[] = {TOP_SRCDIR "/gtksourceview/language-specs", NULL};
-	gchar **lang_dirs;
+	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/data/language-specs", NULL};
+	gchar *builtin_sm_dirs[] = {TOP_SRCDIR "/data/styles", NULL};
+	gchar **dirs;
 	const gchar * const * schemes;
 	gboolean use_default_paths = FALSE;
 
@@ -1479,14 +1604,14 @@ main (int argc, char *argv[])
 // 	gdk_window_set_debug_updates (TRUE);
 
 	/* we do not use defaults so we don't need to install the library */
-	lang_dirs = use_default_paths ? NULL : builtin_lang_dirs;
+	dirs = use_default_paths ? NULL : builtin_lang_dirs;
 	lm = gtk_source_language_manager_get_default ();
-	gtk_source_language_manager_set_search_path (lm, lang_dirs);
+	gtk_source_language_manager_set_search_path (lm, dirs);
 
-	lang_dirs = use_default_paths ? NULL : builtin_sm_dirs;
+	dirs = use_default_paths ? NULL : builtin_sm_dirs;
 
 	sm = gtk_source_style_scheme_manager_get_default ();
-	gtk_source_style_scheme_manager_set_search_path (sm, lang_dirs);
+	gtk_source_style_scheme_manager_set_search_path (sm, dirs);
 
 	if (!use_default_paths)
 		gtk_source_style_scheme_manager_append_search_path (sm, TOP_SRCDIR "/tests/test-scheme.xml");
